/* Elvira Bluetooth FTP 
 * Copyright (C) 2007  Alberto Salas Bragado
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "bftp.h"

static obexftp_client_t *cli= NULL;

int bftp_search(device_t **list) {

	int numdisp;
	device_t *lista;
	
	numdisp = hci_search(&lista);
	*list = lista;
	
	return numdisp;
	
}

int bftp_connect(device_t *device) {

	int ret;

	//Abrimos conexión OBEX Bluetooth
	cli = obexftp_open(OBEX_TRANS_BLUETOOTH, NULL, NULL, NULL);
	if (cli == NULL)
		return E_FTP_LOCAL_CONN;

	//Conectamos con el servidor OBEX FTP
	ret = obexftp_connect(cli, device->mac, device->port); /*!!!*/
	if (ret < 0) {
		obexftp_close(cli);
		cli = NULL;
		return E_FTP_REMOTE_CONN;
	}

	//Conexión sin errores
	return 0;

}

int bftp_disconnect() {

	if (cli != NULL) {
	
		int ret;

		//Desconectamos del servidor OBEX FTP
		ret = obexftp_disconnect(cli);

		//Cerramos la conexion OBEX Bluetooth
		obexftp_close(cli);
		cli = NULL;

		//Devolvemos el resultado de la operación
		if (ret < 0)
			return E_FTP_DISCONNECTING;
	}
	
	return 0;
	
}

int bftp_state() {
	
	if (cli == NULL)
		
		return FTP_DISCONNECTED;
		
	else
		
		return FTP_CONNECTED;
	
}

int bftp_list(char *directory, file_t **list) {

	int tam=0;
	file_t *lista= NULL;
	//Variables de libxml2 necesarias para parsear el documento XML
	xmlDocPtr doc;
	xmlNodePtr cur;

	//Obtenemos el listado de los ficheros del directorio en XML
	if (obexftp_list(cli, NULL, directory) < 0) {
		return E_FTP_LIST;
	}

	//Creamos el objeto Document de libxml2
	doc = xmlReadMemory((const char *) cli->buf_data, (int) cli->buf_size,
			NULL, NULL, 0);
	//Movemos el cursor al principio del documento
	cur = xmlDocGetRootElement(doc);

	//Comprobamos si el documento esta vacío
	if (cur == NULL) {
		fprintf(stderr,"Documento Vacío\n");
		xmlFreeDoc(doc);
		return E_FTP_LIST_EMPTY;
	}

	//Comprobamos que el nodo raiz sea el correcto
	if (xmlStrcmp(cur->name, (const xmlChar *) "folder-listing")) {
		fprintf(stderr,"Documento de tipo incorrecto, root node != folder-listing\n");
		xmlFreeDoc(doc);
		return E_FTP_LIST_CORRUPT;
	}

	//Movemos el cursor al primer hijo
	cur = cur->xmlChildrenNode;

	//Repetimos hasta que no haya mas hijos
	while (cur != NULL) {

		//Comprobamos que el nodo actual sea un folder
		if ((!xmlStrcmp(cur->name, (const xmlChar *) "folder"))) {

			//Incrementamos el tamaño de la lista a devolver
			tam++;
			//Ampliamos el tamaño de la memoria dinámica solicitada para añadir el folder actual
			lista = (file_t *) realloc(lista, sizeof(file_t)*tam);

			//Recuperamos las propiedades del nodo folder actual
			lista[tam-1].type = FILET_FOLDER;
			sprintf(lista[tam-1].name, "%s", xmlGetProp(cur,
					(const xmlChar *) "name"));
			if (xmlGetProp(cur, (const xmlChar *) "created") != NULL) {
				sprintf(lista[tam-1].created, "%s", xmlGetProp(cur,
						(const xmlChar *) "created"));
			}
			sprintf(lista[tam-1].user_perm, "%s", xmlGetProp(cur,
					(const xmlChar *) "user-perm"));
			sprintf(lista[tam-1].mem_type, "%s", xmlGetProp(cur,
					(const xmlChar *) "mem-type"));
			strcpy(lista[tam-1].size, "");

			//Si no, comprobamos que el nodo actual sea un file
		} else if ((!xmlStrcmp(cur->name, (const xmlChar *) "file"))) {

			//Incrementamos el tamaño de la lista a devolver
			tam++;
			//Ampliamos el tamaño de la memoria dinámica solicitada para añadir el file actual
			lista = (file_t *) realloc(lista, sizeof(file_t)*tam);

			//Recuperamos las propiedades del nodo file actual
			lista[tam-1].type = FILET_FILE;
			sprintf(lista[tam-1].name, "%s", xmlGetProp(cur,
					(const xmlChar *) "name"));
			sprintf(lista[tam-1].size, "%s", xmlGetProp(cur,
					(const xmlChar *) "size"));
			sprintf(lista[tam-1].modified, "%s", xmlGetProp(cur,
					(const xmlChar *) "modified"));
			sprintf(lista[tam-1].user_perm, "%s", xmlGetProp(cur,
					(const xmlChar *) "user-perm"));

		}

		//Nos movemos al siguiente nodo
		cur = cur->next;

	}

	//Liberamos el objeto Document de libxml2
	xmlFreeDoc(doc);

	//Devolvemos la lista de ficheros creada
	*list = lista;
	//Devolvemos el tamaño de lista de ficheros
	return tam;
}

int bftp_put(char *local_route, char *remote_route) {

	int ret;

	//Mensaje debug
	//printf("\n* Subiendo %s a %s ... OK!\n", local_route, remote_route);

	//Solicitamos el envío de un fichero al servidor y comprobamos la respuesta
	ret = obexftp_put_file(cli, local_route, remote_route);
	if (ret < 0) {
		return E_FTP_PUT;
	}

	//Volvemos al directorio raiz
	//Es una "fallo" de obexftp
	obexftp_cdtop(cli);

	//Devolvemos el resultado de la operacion
	return (ret);
}

int bftp_get(char *local_route, char *remote_route) {

	int ret;

	//Mensaje debug
	//printf("\n* Descargando %s a %s ... OK!\n", remote_route, local_route);

	//Solicitamos la recepción de un fichero desde el servidor
	ret = obexftp_get(cli, local_route, remote_route);
	if (ret < 0) {
		return E_FTP_GET;
	}

	//Volvemos al directorio raiz
	//Es una "fallo" de obexftp
	obexftp_cdtop(cli);

	//Devolvemos el resultado de la operación
	return (0);
}

int bftp_del_file(char *remote_route) {

	int ret;

	//Solicitamos la eliminación del fichero remoto
	ret = obexftp_del(cli, remote_route);
	if (ret < 0) {
		return E_FTP_DEL;
	}

	//Volvemos al directorio raiz
	//Es una "fallo" de obexftp
	obexftp_cdtop(cli);

	//Devolvemos el resultado de la operación
	return (ret);
}

int bftp_del_dir(char *remote_route) {

	int nficheros;
	file_t *lista;
	
	nficheros = bftp_list(remote_route, &lista);
	free(lista);
	
	if (nficheros == 0)
		
		return bftp_del_file(remote_route);
	
	return E_FTP_DEL;
	
}

int bftp_mkdir(char *remote_route) {

	int ret;

	//Solicitamos la creación del directorio en el servidor
	ret = obexftp_mkpath(cli, remote_route);
	if (ret < 0) {
		return E_FTP_DEL;
	}

	//Volvemos al directorio raiz
	//Es una "fallo" de obexftp
	obexftp_cdtop(cli);

	//Devolvemos el resultado de la operación
	return (ret);
}

void bftp_free(void **pointer) {

	//Liberamos la memoria dinámica
	free(*pointer);

	//Puntero a null para evitar confusiones
	*pointer = NULL;

}

int bftp_file_exist(char *remote_route) {
	if (obexftp_stat(cli, remote_route) == NULL)
		return E_FTP_EXIST;
	return 0;
}
