/* Elvira Bluetooth FTP 
 * Copyright (C) 2007  Alberto Salas Bragado
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef BFTP_H_
#define BFTP_H_

#include "btsearch.h"
#include "types.h"

#include <stdio.h>
#include <stdlib.h>
#include <obexftp/client.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

/**
 * Constantes que definen si hay conexión o no
 */
#define FTP_CONNECTED 1
#define FTP_DISCONNECTED 0

/**
 * Constantes de error
 */
#define E_LOCAL_DISP -1
#define E_SEARCH_DISP -2
#define E_FTP_LOCAL_CONN -3
#define E_FTP_REMOTE_CONN -4
#define E_FTP_DISCONNECTING -5
#define E_FTP_DISCONNECTED -6
#define E_FTP_LIST -7
#define E_FTP_LIST_EMPTY -8
#define E_FTP_LIST_CORRUPT -9
#define E_FTP_GET -10
#define E_FTP_PUT -11
#define E_FTP_DEL -12
#define E_FTP_MKDIR -13
#define E_FTP_EXIST -14

/**
 * Busqueda de dispositivos que exportan Obex FTP
 * OUT list Array de dispositivos encontrados
 * RETURN Número de dispositivos encontrados o <0 si error
 */
int bftp_search(device_t **list);

/**
 * Conectar con un dispositivo a través de OBEX FTP
 * IN device Dispositivo al que conectarse
 * RETURN identificador de conexión, o <0 si error
 */
int bftp_connect(device_t *device);

/**
 * Desconectar de un dispositivo
 * RETURN <0 si hay algún error
 */
int bftp_disconnect();

/**
 * Comprobar si existe conexión
 * RETURN constante de conexion
 */
int bftp_state();

/**
 * Listar el contenido de una carpeta
 * IN directory Nombre del directorio a listar
 * OUT list Array de ficheros que contiene el directorio
 * RETURN Numero de ficheros que contiene el directorio o <0 si error
 */
int bftp_list(char *directory, file_t **list);

/**
 * Envía un fichero al dispositivo
 * IN local_route Ruta y nombre del fichero a enviar
 * IN remote_route Ruta y nombre del fichero en el dispositivo
 * RETURN <0 si error
 */
int bftp_put(char *local_route, char *remote_route);

/**
 * Recibe un fichero desde el dispositivo
 * IN local_route Ruta y nombre del fichero local
 * IN remote_route Ruta y nombre del fichero en el dispositivo
 * RETURN <0 si error
 */
int bftp_get(char *local_route, char *remote_route);

/**
 * Elimina un fichero remoto
 * IN remote_route Ruta y nombre del fichero en el dispositivo a borrar
 * RETURN <0 si error
 */
int bftp_del_file(char *remote_route);

/**
 * Elimina un directorio remoto
 * IN remote_route Ruta y nombre del directorio en el dispositivo a borrar
 * RETURN <0 si error
 */
int bftp_del_dir(char *remote_route);

/**
 * Crea un directorio en el servidor
 * IN remote_route Ruta del directorio a crear
 * RETURN <0 si error
 */
int bftp_mkdir(char *remote_route);

/**
 * Comprueba si un fichero o directorio existe
 * IN remote_route Ruta del directorio o fichero a comprobar
 * RETURN 0 si existe, <0 si no existe
 */
int bftp_file_exist(char *remote_route);

/**
 * Liberar recursos solicitados
 * IN pointer Puntero a los recursos solicitados
 */
void bftp_free(void **pointer);

#endif /*BFTP_H_*/
