/* Elvira Bluetooth FTP 
 * Copyright (C) 2007  Alberto Salas Bragado
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "callbacks.h"

/** Cadenas con las rutas actuales del sistema local y remoto */
char local_route[MAXPATHLEN];
char remote_route[MAXPATHLEN];

/** Declaración de funciones útiles del módulo */
void remote_reload();
void local_reload();

/** Callbacks de la ventana principal */

void on_mainWindow_destroy() {
	device_t *favoritos;
	int numfavoritos;
	
	g_print("donpi tony!\n");
	
	//Desconectar
	if (bftp_disconnect() < 0)
		g_print("Ocurrio un error al cerrar la conexion con el dispositivo. ¿Había conexión?\n");
	
	//Extraer favoritos y escribirlos
	getDevicesGTV(lstFavoritos, &favoritos, &numfavoritos);
	writeFavorites(favoritos, numfavoritos);
	
	free(favoritos);
	
	gtk_main_quit();
}

void on_btnConectar_clicked(GtkToolButton *button, gpointer user_data) {

	if (!gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(button))) {
		l("Intentamos desconexión del dispositivo");
		disconnectProcessGui();
		return;
	}

	//Mostramos la ventana
	gtk_widget_show((GtkWidget *)connWindow);

}

void on_lstRemote_row_activated(GtkTreeView *treeview, GtkTreePath *path,
		GtkTreeViewColumn *col, gpointer userdata) {

	file_t f;
	char logm[200];

	//Obtenemos el fichero seleccionado
	getFileActivated(treeview, path, &f);

	//En caso de que se haya pulsado sobre ".."
	if (strcmp(f.name, "..") == 0) {

		char *posicion;
				
		//Cambiamos la ruta remota
		posicion = strrchr(remote_route, '/');
		if (posicion == remote_route)
			posicion++;
		*posicion = '\0';

		//Log
		sprintf(logm, "Accedemos a la ruta remota %s", remote_route);
		l(logm);

		//Recargar la lista remota
		remote_reload();

	} else if (f.type == FILET_FOLDER) {
		
		//Cambiamos la ruta remota
		if (strcmp(remote_route, "/") != 0)
			strcat(remote_route, "/");
		strcat(remote_route, f.name);

		//Log
		sprintf(logm, "Accedemos a la ruta remota %s", remote_route);
		l(logm);
		
		//Recargar la lista remota
		remote_reload();

	} else {
		
		char temp_route[MAXPATHLEN];
		char temp2_route[MAXPATHLEN];
		
		//Lo primero es comprobar si se tienen permisos para escribir en el directorio local
		if (local_wperm(local_route) != 0) {
			sprintf(logm, "No tiene permisos para escribir en el directorio %s", local_route);
			l(logm);
			return;
		}

		//Generamos la ruta del fichero remoto
		strcpy(temp_route, remote_route);
		if (strcmp(temp_route, "/") != 0)
			strcat(temp_route, "/");
		strcat(temp_route, f.name);
		
		//Generamos la ruta del fichero local
		strcpy(temp2_route, local_route);
		if (strcmp(temp2_route, "/") != 0)
			strcat(temp2_route, "/");
		strcat(temp2_route, f.name);
		
		//Comprobamos si el fichero existe ya
		if (local_file_exist(temp2_route) >= 0) {
			l("El fichero existe ya en el directorio local.");
			return;
		}

		//Log
		sprintf(logm, "Comienza la descarga de %s a %s", temp_route, temp2_route);
		l(logm);
		
		//Mostramos el dialogo de transferencia
		gtk_label_set_text(lblProgress, "(1/1)");
		gtk_label_set_text(lblFile, f.name);
		gtk_widget_show((GtkWidget *)progressDialog);
		
		//Actualizar GUI
		updateGUI();				

		//Descargamos el fichero
		if (bftp_get(temp2_route, temp_route) >= 0) {
		
			//Log
			sprintf(logm, "Fichero %s descargado correctamente", temp_route);
			l(logm);

			//Recargar la lista local
			local_reload();

		} else {

			//Log
			sprintf(logm, "ERROR: No se pudo descargar el fichero %s", temp_route);
			l(logm);

		}
		
		//Ocultamos el dialogo de transferencia
		gtk_widget_hide((GtkWidget *)progressDialog);
	}
	
	//Actualizamos el texto de la GUI con la nueva ruta remota
	gtk_entry_set_text(txtRemote, remote_route);
	
}


void on_lstLocal_row_activated(GtkTreeView *treeview, GtkTreePath *path,
		GtkTreeViewColumn *col, gpointer userdata) {

	file_t f;
	char logm[200];
	char temp_route[MAXPATHLEN];

	//Obtenemos el fichero seleccionado
	getFileActivated(treeview, path, &f);

	//En caso de que se haya pulsado sobre ".."
	if (strcmp(f.name, "..") == 0) {

		char *posicion;
		
		//Calculamos la posible ruta destino
		strcpy(temp_route, local_route);
		posicion = strrchr(temp_route, '/');
		if (posicion == temp_route)
			posicion++;
		*posicion = '\0';
		
		//Comprobamos si tenemos permiso de lectura sobre el directorio
		if (local_rperm(temp_route) < 0) {

			//Log
			l("No tiene permiso para acceder al directorio");
			return;
			
		}
			
		//Cambiamos a la nueva ruta local
		strcpy(local_route, temp_route);

		//Log
		sprintf(logm, "Accedemos a la ruta local %s", local_route);
		l(logm);

		//Recargar la lista local
		local_reload();
			
	} else if (f.type == FILET_FOLDER) {
		
		//Cambiamos la ruta local
		strcpy(temp_route, local_route);
		if (strcmp(temp_route, "/") != 0)
			strcat(temp_route, "/");
		strcat(temp_route, f.name);
		
		//Comprobamos si tenemos permiso de lectura sobre el directorio
		if (local_rperm(temp_route) < 0) {

			//Log
			l("No tiene permiso para acceder al directorio");
			return;
					
		}
			
		//Cambiamos a la nueva ruta local
		strcpy(local_route, temp_route);

		//Log
		sprintf(logm, "Accedemos a la ruta local %s", local_route);
		l(logm);
		
		//Recargar la lista local
		local_reload();

	} else {
		
		char temp2_route[MAXPATHLEN];
		
		//Generamos la ruta del fichero local
		strcpy(temp2_route, local_route);
		if (strcmp(temp2_route, "/") != 0)
			strcat(temp2_route, "/");
		strcat(temp2_route, f.name);
		
		//Lo primero es comprobar si se tienen permisos para leer el fichero local
		if (local_rperm(temp2_route) != 0) {
			sprintf(logm, "No tiene permisos para leer el fichero %s", temp2_route);
			l(logm);
			return;
		}

		//Generamos la ruta del fichero remoto
		strcpy(temp_route, remote_route);
		if (strcmp(temp_route, "/") != 0)
			strcat(temp_route, "/");
		strcat(temp_route, f.name);
		
		//Comprobamos si el fichero existe ya
		if (bftp_file_exist(temp_route) >= 0) {
			l("El fichero existe ya en el directorio remoto.");
			return;
		}

		//Log
		sprintf(logm, "Comienza el envío de %s a %s", temp2_route, temp_route);
		l(logm);
		
		//Mostramos el dialogo de transferencia
		gtk_label_set_text(lblProgress, "(1/1)");
		gtk_label_set_text(lblFile, f.name);
		gtk_widget_show((GtkWidget *)progressDialog);
				
		//Actualizar GUI
		updateGUI();

		//Descargamos el fichero
		if (bftp_put(temp2_route, temp_route) >= 0) {
			
			//Log
			sprintf(logm, "Fichero %s enviado correctamente", temp2_route);
			l(logm);

			//Recargar la lista remota
			remote_reload();

		} else {

			//Log
			sprintf(logm, "ERROR: No se pudo enviar el fichero %s", temp2_route);
			l(logm);

		}
		
		//Ocultamos el dialogo de transferencia
		gtk_widget_hide((GtkWidget *)progressDialog);
		
	}
	
	//Actualizamos el texto de la GUI con la nueva ruta local
	gtk_entry_set_text(txtLocal, local_route);
	
}

void on_btnGet_clicked(GtkButton *button, gpointer user_data) {
	
	file_t *files;
	int numfiles, indice;
	char temp_route[MAXPATHLEN];
	char temp2_route[MAXPATHLEN];
	char logm[400];
	
	//Lo primero es comprobar si se tienen permisos para escribir en el directorio local
	if (local_wperm(local_route) != 0) {
		l("No tiene permisos para escribir en el directorio local");
		return;
	}

	//Obtenemos la selección remota
	getFilesSelected(lstRemote, &files, &numfiles);
	
	//Recorremos la lista de ficheros seleccionados
	for (indice = 0; indice < numfiles; indice++) {
		
		//Comprobamos si es un directorio
		if (files[indice].type == FILET_FOLDER) {
			sprintf(logm, "No se permite transferir directorios (%s)", files[indice].name);
			l(logm);
			continue;
		}
		
		//Generamos la ruta del fichero remoto
		strcpy(temp_route, remote_route);
		if (strcmp(temp_route, "/") != 0)
			strcat(temp_route, "/");
		strcat(temp_route, files[indice].name);

		//Generamos la ruta del fichero local
		strcpy(temp2_route, local_route);
		if (strcmp(temp2_route, "/") != 0)
			strcat(temp2_route, "/");
		strcat(temp2_route, files[indice].name);
		
		//Comprobamos si el fichero existe ya en el directorio local
		if (local_file_exist(temp2_route) >= 0) {
			sprintf(logm, "El fichero %s existe ya en el directorio local.", files[indice].name);
			l(logm);
			continue;
		}

		//Log
		sprintf(logm, "Comienza la descarga de %s a %s", temp_route, temp2_route);
		l(logm);
		
		//Mostramos el dialogo de transferencia
		sprintf(logm, "(%d/%d)", indice+1, numfiles);
		gtk_label_set_text(lblProgress, logm);
		gtk_label_set_text(lblFile, files[indice].name);
		gtk_widget_show((GtkWidget *)progressDialog);
				
		//Actualizar GUI
		updateGUI();
		
		//Descargamos el fichero
		if (bftp_get(temp2_route, temp_route) >= 0) {
		
			//Log
			sprintf(logm, "Fichero %s descargado correctamente", temp_route);
			l(logm);
			
			//Recargar la lista local
			local_reload();

		} else {

			//Log
			sprintf(logm, "ERROR: No se pudo descargar el fichero %s", temp_route);
			l(logm);

		}
		
	}
	
	gtk_widget_hide((GtkWidget *)progressDialog);
	
	//Liberamos la lista de seleccionados
	free(files);
	
}

void on_btnPut_clicked(GtkButton *button, gpointer user_data) {
	
	file_t *files;
	int numfiles, indice;
	char temp_route[MAXPATHLEN];
	char temp2_route[MAXPATHLEN];
	char logm[400];
	
	//Obtenemos la selección remota
	getFilesSelected(lstLocal, &files, &numfiles);
	
	//Recorremos la lista de ficheros seleccionados
	for (indice = 0; indice < numfiles; indice++) {
		
		//Generamos la ruta del fichero remoto
		strcpy(temp_route, remote_route);
		if (strcmp(temp_route, "/") != 0)
			strcat(temp_route, "/");
		strcat(temp_route, files[indice].name);
		
		//Comprobamos si el fichero existe ya en el directorio remoto
		if (bftp_file_exist(temp_route) >= 0) {
			sprintf(logm, "El fichero %s existe ya en el directorio remoto.", files[indice].name);
			l(logm);
			continue;
		}

		//Generamos la ruta del fichero local
		strcpy(temp2_route, local_route);
		if (strcmp(temp2_route, "/") != 0)
			strcat(temp2_route, "/");
		strcat(temp2_route, files[indice].name);
		
		//Comprobamos si es un directorio
		if (files[indice].type == FILET_FOLDER) {
			sprintf(logm, "No se permite transferir directorios (%s)", files[indice].name);
			l(logm);
			continue;
		//Comprobamos si se tiene permisos
		} else if (local_rperm(temp2_route) != 0) {
			sprintf(logm, "No tiene permisos para leer el fichero local %s", temp2_route);
			l(logm);
			continue;
		}

		//Log
		sprintf(logm, "Comienza el envio de %s a %s", temp_route, temp2_route);
		l(logm);
		
		//Mostramos el dialogo de transferencia
		sprintf(logm, "(%d/%d)", indice+1, numfiles);
		gtk_label_set_text(lblProgress, logm);
		gtk_label_set_text(lblFile, files[indice].name);
		gtk_widget_show((GtkWidget *)progressDialog);
				
		//Actualizar GUI
		updateGUI();
		
		//Enviamos el fichero
		if (bftp_put(temp2_route, temp_route) >= 0) {
		
			//Log
			sprintf(logm, "Fichero %s enviado correctamente", temp2_route);
			l(logm);
			
			//Recargar la lista remota
			remote_reload();

		} else {

			//Log
			sprintf(logm, "ERROR: No se pudo enviar el fichero %s", temp2_route);
			l(logm);

		}
		
	}
	
	gtk_widget_hide((GtkWidget *)progressDialog);
	
	//Liberamos la lista de seleccionados
	free(files);
	
}

void on_btnDirLocal_clicked(GtkToolButton *button, gpointer user_data) {
		
	char logm[200];

	//comprobar si estamos conectados
	if (bftp_state() != FTP_CONNECTED) {
		return;
	//comprobar si se tienen permisos para escribir en el directorio local
	} else if (local_wperm(local_route) != 0) {
		sprintf(logm, "No tiene permisos para escribir en el directorio %s", local_route);
		l(logm);
		return;
	}

	g_signal_handlers_disconnect_by_func((GObject *) newDirDialog,
			(GCallback) on_newDirRemote_response, NULL);
	g_signal_connect((GObject *) newDirDialog, "response",
			(GCallback) on_newDirLocal_response, NULL);
	gtk_widget_show((GtkWidget *)newDirDialog);

}

void on_btnDirRemote_clicked(GtkToolButton *button, gpointer user_data) {
	
	if (bftp_state() != FTP_CONNECTED)
		return;
	
	g_signal_handlers_disconnect_by_func((GObject *) newDirDialog,
			(GCallback) on_newDirLocal_response, NULL);
	g_signal_connect((GObject *) newDirDialog, "response",
			(GCallback) on_newDirRemote_response, NULL);
	gtk_widget_show((GtkWidget *)newDirDialog);
	
}

void on_btnDelRemote_clicked(GtkToolButton *button, gpointer user_data) {
	
	if (bftp_state() != FTP_CONNECTED)
		return;
	
	g_signal_handlers_disconnect_by_func((GObject *) delDialog,
			(GCallback) on_delLocalDialog_response, NULL);
	g_signal_connect((GObject *) delDialog, "response",
			(GCallback) on_delRemoteDialog_response, NULL);
	gtk_widget_show((GtkWidget *)delDialog);
	
}

void on_btnDelLocal_clicked(GtkToolButton *button, gpointer user_data) {
	
	if (bftp_state() != FTP_CONNECTED)
		return;
	
	g_signal_handlers_disconnect_by_func((GObject *) delDialog,
			(GCallback) on_delRemoteDialog_response, NULL);
	g_signal_connect((GObject *) delDialog, "response",
			(GCallback) on_delLocalDialog_response, NULL);
	gtk_widget_show((GtkWidget *)delDialog);
	
}

void on_btnAbout_clicked(GtkToolButton *button, gpointer user_data) {
	
	//Mostramos la ventana
	gtk_widget_show((GtkWidget *)aboutDialog);
	
}

/** Callbacks de la ventana de conexión */
void on_connWindow_delete_event() {
	gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(btnConectar),
			FALSE);
	gtk_widget_hide((GtkWidget *)connWindow);
}

void on_btnBusquedaBuscar_clicked(GtkButton *button, gpointer user_data) {

	device_t *lista_dispositivos;
	int ndevices;
	char logm[200];

	l("Comienza una búsqueda de dispositivos");

	ndevices = bftp_search(&lista_dispositivos);
	
	if (ndevices < 0) {
		if (ndevices == E_LOCAL_DISP) {
			l("ERROR: No pudo accederse al dispositivo Bluetooth");
		} else if(ndevices == E_SEARCH_DISP) {
			l("ERROR: Ocurrio un error en a b�squeda de dispositivos");
		} else {
			l("ERROR: Ocurrio un error al lanzar la b�squeda de dispositivos");
		}
		return;
	}

	sprintf(logm, "Finalizada búsqueda de dispositivos: %d encontrados",
			ndevices);
	l(logm);

	poblateDeviceList(lstBusqueda, lista_dispositivos, ndevices);
	
	free(lista_dispositivos);
	
}

void on_btnBusquedaConectar_clicked(GtkButton *button, gpointer user_data) {
	
	char logm[200];
	device_t disp;
	int resultado;
	
	strcpy(disp.mac, "");

	getDeviceSelected(lstBusqueda, &disp);
	
	if (strcmp(disp.mac, "") == 0)
		return;
	
	sprintf(logm, "Solicitamos la conexión con el dispositivo %s puerto %d", disp.mac, disp.port);
	l(logm);

	resultado = bftp_connect(&disp);
	
	if (resultado >= 0) {
		
		l("Conexión aceptada");

		//Construir las rutas iniciales
		strcpy(local_route, getenv("HOME"));
		strcpy(remote_route, "/");
		
		//Rellenar los campos de texto con las rutas
		gtk_entry_set_text(txtLocal, local_route);
		gtk_entry_set_text(txtRemote, remote_route);

		local_reload();
		remote_reload();

		//Ocultar la ventana de conexión
		gtk_widget_hide((GtkWidget *)connWindow);

	} else {

		if (resultado == E_FTP_LOCAL_CONN) {
			l("ERROR: Ocurrio un error al acceder al dispositivo local");
		} else {
			l("No se pudo realizar la conexion");
		}
		
		//Mostramos la ventana
		gtk_widget_show((GtkWidget *)errorDialog);
	}
	
}

void on_btnFavoritosConectar_clicked(GtkButton *button, gpointer user_data) {

	char logm[200];
	device_t disp;
	int resultado;

	strcpy(disp.mac, "");
	
	getDeviceSelected(lstFavoritos, &disp);

	if (strcmp(disp.mac, "") == 0)
		return;
	
	sprintf(logm, "Solicitamos la conexión con el dispositivo %s puerto %d", disp.mac, disp.port);
	l(logm);
	
	resultado = bftp_connect(&disp);
		
	if (resultado >= 0) {
		
		l("Conexión aceptada");

		//Construir las rutas iniciales
		strcpy(local_route, getenv("HOME"));
		strcpy(remote_route, "/");
		
		//Rellenar los campos de texto con las rutas
		gtk_entry_set_text(txtLocal, local_route);
		gtk_entry_set_text(txtRemote, remote_route);

		local_reload();
		remote_reload();

		//Ocultar la ventana de conexión
		gtk_widget_hide((GtkWidget *)connWindow);

	} else {

		if (resultado == E_FTP_LOCAL_CONN) {
			l("ERROR: Ocurrio un error al acceder al dispositivo local");
		} else {
			l("No se pudo realizar la conexion");
		}
		
		//Mostramos la ventana
		gtk_widget_show((GtkWidget *)errorDialog);
	}

}

void on_btnAddFavorite_clicked(GtkButton *button, gpointer user_data) {
	
	device_t disp;
	
	strcpy(disp.mac, "");
	
	//obtener dispositivo seleccionado
	getDeviceSelected(lstBusqueda, &disp);
	
	if (strcmp(disp.mac, "") != 0) {
		//añadir dispositivo a lista
		addDeviceList(lstFavoritos, &disp);
	}
	
}

void on_btnDelFavorite_clicked(GtkButton *button, gpointer user_data) {
	
	//eliminar de la lista de favoritos el dispositivo seleccionado
	delDeviceList(lstFavoritos);
	
}

/* Callbacks de la ventana de error de conexión */
void on_errorDialog_delete_event() {
	gtk_widget_hide((GtkWidget *)errorDialog);
}

void on_btnErrorOK_clicked(GtkButton *button, gpointer user_data) {
	gtk_widget_hide((GtkWidget *)errorDialog);
}

/* Callbacks de la ventana de confirmación de eliminacion de fichero remoto */
void on_delRemoteDialog_response(GtkDialog * dialogo, gint response, gpointer data) {
	
	file_t *files;
	int nfiles, indice;
	char temp_route[MAXPATHLEN];
	char logm[200];

	switch (response) {
		case GTK_RESPONSE_OK:
			getFilesSelected(lstRemote, &files, &nfiles);
			
			for (indice = 0; indice < nfiles; indice++) {
				
				//Generamos la ruta del fichero remoto
				strcpy(temp_route, remote_route);
				if (strcmp(temp_route, "/") != 0)
					strcat(temp_route, "/");
				strcat(temp_route, files[indice].name);
						
				if ((files[indice].type == FILET_FILE) && (bftp_del_file(temp_route) >= 0)) {
					sprintf(logm, "El fichero %s se eliminó correctamente.", temp_route);
					l(logm);
				} else if ((files[indice].type == FILET_FOLDER) && (bftp_del_dir(temp_route) >= 0)) {
					sprintf(logm, "El directorio %s se eliminó correctamente.", temp_route);
					l(logm);
				} else {
					sprintf(logm, "Ocurrió un error al eliminar %s", temp_route);
					l(logm);
				}
			}
			
			free(files);
			break;
		case GTK_RESPONSE_CANCEL:
			break;
		default:
			break;
	}
	remote_reload();
	gtk_widget_hide((GtkWidget *)delDialog);
	
}

void on_delLocalDialog_response(GtkDialog * dialogo, gint response, gpointer data) {

	file_t *files;
	int nfiles, indice;
	char temp_route[MAXPATHLEN];
	char logm[200];

	switch (response) {
		case GTK_RESPONSE_OK:
			getFilesSelected(lstLocal, &files, &nfiles);
			
			for (indice = 0; indice < nfiles; indice++) {
				
				//Generamos la ruta del fichero local
				strcpy(temp_route, local_route);
				if (strcmp(temp_route, "/") != 0)
					strcat(temp_route, "/");
				strcat(temp_route, files[indice].name);
						
				if ((files[indice].type == FILET_FILE) && (local_del_file(temp_route) >= 0)) {
					sprintf(logm, "El fichero %s se eliminó correctamente.", temp_route);
					l(logm);
				} else if ((files[indice].type == FILET_FOLDER) && (local_del_dir(temp_route) >= 0)) {
					sprintf(logm, "El directorio %s se eliminó correctamente.", temp_route);
					l(logm);
				} else {
					sprintf(logm, "Ocurrió un error al eliminar %s", temp_route);
					l(logm);
				}
			}
			
			free(files);
			break;
		case GTK_RESPONSE_CANCEL:
			break;
		default:
			break;
	}
	local_reload();
	gtk_widget_hide((GtkWidget *)delDialog);

}

void on_delDialog_delete_event() {
	gtk_widget_hide((GtkWidget *)delDialog);
}

/* Callbacks de la ventana de confirmación de eliminacion de fichero local */
void on_newDirRemote_response(GtkDialog * dialogo, gint response, gpointer data) {

	char nombre[255];
	char logm[200];
	char temp_route[MAXPATHLEN];

	switch (response) {
		case GTK_RESPONSE_OK:
			//obtenemos lo que tecleo el usuario
			getTextEntry(txtNewDir, nombre);
			
			if (strcmp(nombre, "") != 0) {
				
				//Generamos la ruta del directorio remoto
				strcpy(temp_route, remote_route);
				if (strcmp(temp_route, "/") != 0)
					strcat(temp_route, "/");
				strcat(temp_route, nombre);
				
				//comprobar si ya existe
				if (bftp_file_exist(temp_route) == 0) {
									
					sprintf(logm, "El directorio %s ya existe en la ruta actual", nombre);
					l(logm);
								
				//crear el directorio
				} else if (bftp_mkdir(nombre) < 0) {
					
					sprintf(logm, "Ocurrio un error al crear el directorio remoto %s, compruebe que no exista ya", nombre);
					l(logm);
					
				} else {
					
					sprintf(logm, "El directorio remoto %s se creo sin problemas", nombre);
					l(logm);
					remote_reload();
					
				}
			}
				
			break;
		case GTK_RESPONSE_CANCEL:
			break;
		default:
			break;
	}
	
	gtk_entry_set_text(txtNewDir, "");
	gtk_widget_hide((GtkWidget *)newDirDialog);
	
}

void on_newDirLocal_response(GtkDialog * dialogo, gint response, gpointer data) {

	char nombre[255];
	char logm[200];
	char temp_route[MAXPATHLEN];

	switch (response) {
		case GTK_RESPONSE_OK:
			//obtenemos lo que el usuario tecleo
			getTextEntry(txtNewDir, nombre);
			
			if (strcmp(nombre, "") != 0) {
				
				//Generamos la ruta del directorio local
				strcpy(temp_route, local_route);
				if (strcmp(temp_route, "/") != 0)
					strcat(temp_route, "/");
				strcat(temp_route, nombre);
				
				//comprobar si ya existe
				if (local_file_exist(temp_route) == 0) {
					
					sprintf(logm, "El directorio %s ya existe en la ruta actual", nombre);
					l(logm);
				
				//crear el directorio
				} else if (local_mkdir(temp_route) < 0) {
					
					sprintf(logm, "Ocurrio un error al crear el directorio local %s, compruebe que el directorio no existe ya", nombre);
					l(logm);
					
				} else {
					
					sprintf(logm, "El directorio local %s se creo sin problemas", nombre);
					l(logm);
					local_reload();
					
				}
			}
			break;
		case GTK_RESPONSE_CANCEL:
			break;
		default:
			break;
	}
	
	gtk_entry_set_text(txtNewDir, "");
	gtk_widget_hide((GtkWidget *)newDirDialog);
	
}

void on_newDirDialog_delete_event() {
	gtk_widget_hide((GtkWidget *)newDirDialog);
}

void on_txtNewDir_activate() {
	//on_newDirRemote_response(newDirDialog, GTK_RESPONSE_OK, NULL);
}

/* Callbacks de la ventana de progreso de transferencia */
void on_progressDialog_delete_event() {
	return;
}

/* Callbacks del dialogo acerca de... */
void on_aboutDialog_delete_event() {
	gtk_widget_hide((GtkWidget *)aboutDialog);
}

/** Funciones útiles del modulo */
void remote_reload() {
	
	file_t *files;
	int root, numfiles;
	
	//Comprobamos si estamos en la ruta local raiz
	root = strcmp(remote_route, "/") == 0 ? 1 : 0;
	//Obtenemos el listado de ficheros de la ruta local
	numfiles = bftp_list(remote_route, &files);
	//Rellenamos la lista local con los nuevos ficheros
	poblateFileList(lstRemote, files, numfiles, root);
	//Liberamos la memoria del listado de ficheros
	free(files);
	
}

void local_reload() {
	
	file_t *files;
	int root, numfiles;
	
	//Comprobamos si estamos en la ruta local raiz
	root = strcmp(local_route, "/") == 0 ? 1 : 0;
	//Obtenemos el listado de ficheros de la ruta local
	numfiles = local_list(local_route, &files);
	//Rellenamos la lista local con los nuevos ficheros
	poblateFileList(lstLocal, files, numfiles, root);
	//Liberamos la memoria del listado de ficheros
	free(files);
	
}
